import { API_URL } from '$env/static/private';
import { error, redirect } from '@sveltejs/kit';
import type { Actions, PageServerLoad } from './$types';

import { superValidate } from 'sveltekit-superforms';
import { zod } from 'sveltekit-superforms/adapters';

import { compile } from 'mdsvex';

import type { Chapter, Puzzle } from '$lib/types';
import { handleRedirect } from '$lib/utils';

import { formSchema } from './schema';

export const load: PageServerLoad = async ({ url, locals: { user }, fetch, cookies, params: { chapterId, puzzleId } }) => {
	if (!user) redirect(302, handleRedirect('login', url));

	const session = cookies.get('session');

	if (isNaN(parseInt(puzzleId))) {
		redirect(302, `/chapters/${chapterId}`);
	}

	let res = await fetch(`${API_URL}/chapter/${chapterId}`);

	if (!res.ok) {
		redirect(302, `/chapters`);
	}

	const chapter: Chapter = await res.json();

	if (!chapter || !chapter.show) {
		redirect(302, `/chapters`);
	}

	if (
		!chapter.puzzles.some((puzzle) => puzzle.id === parseInt(puzzleId)) ||
		!chapter.puzzles.find((puzzle) => puzzle.id === parseInt(puzzleId))?.show
	) {
		redirect(302, `/chapters/${chapterId}`);
	}

	res = await fetch(`${API_URL}/puzzle/${puzzleId}`);

	if (!res.ok) {
		error(404, {
			errorId: 'puzzle_not_found',
			message: 'Puzzle not found'
		});
	}

	const puzzle: Puzzle = await res.json();

	if (!puzzle) {
		error(404, {
			errorId: 'puzzle_not_found',
			message: 'Puzzle not found'
		});
	}

	const content = await compile(puzzle.content);

	if (content)
		puzzle.content = content?.code
			.replace(/>{@html `<code class="language-/g, '><code class="language-')
			.replace(/<\/code>`}<\/pre>/g, '</code></pre>');

	return {
		title: `${chapter.name} - ${puzzle.name}`,
		puzzle,
		url: `${API_URL}/puzzleResponse/${puzzleId}`,
		session,
		form: await superValidate(zod(formSchema))
	};
}

export const actions: Actions = {
	default: async ({ request, fetch, params: { chapterId, puzzleId } }) => {

		const formData = await request.formData();

		console.log(formData)

		const res = await fetch(`${API_URL}/puzzleResponse/${puzzleId}`, {
			method: 'POST',
			body: formData
		});

		if (!res.ok) {
			console.log(res)
			return;
		}

		redirect(302, `/chapters/${chapterId}/puzzle/${puzzleId}`);
	}
}
